/*REXX **************************************************************/
/*                                                                  */
/* Program name: DOCTMLT                                            */
/*                                                                  */
/* Description: CICS TS for OS/390 and z/OS - A utility to convert  */
/*              DOCUMENT templates to data only programs for        */
/*              improved  performance.                              */
/*                                                                  */
/*  Statement:  Licensed Materials - Property of IBM                */
/*              IBM CICS SupportPac CA8I                            */
/*              (C) Copyright IBM Corp. 2002                        */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Change history:                                                  */
/*                                                                  */
/* V1.1   13-Dec-2002  Initial release                              */
/*                     Authors Mark Cocker & Peter Havercan         */
/*                                                                  */
/********************************************************************/

/* retrieve arguments */
PARSE SOURCE operatingSystem rest
IF operatingSystem = 'CMS' THEN
   DO
   PARSE UPPER ARG INFILE.name INFILE.type INFILE.mode OUTFILE.name,
                   OUTFILE.type OUTFILE.mode csectName options
   INFILE.name = INFILE.name INFILE.type INFILE.mode
   OUTFILE.name = OUTFILE.name OUTFILE.type OUTFILE.mode
   END
ELSE
   PARSE UPPER ARG INFILE.name OUTFILE.name csectName options

IF INFILE.name = '?' THEN
   SIGNAL HelpDocumentation

/* validate input filename */
IF LENGTH(INFILE.name) = 0 THEN
   DO
   SAY 'Please enter input file:'

   IF operatingSystem = 'CMS' THEN
      DO
      PARSE UPPER PULL INFILE.name
      IF LENGTH(INFILE.name) > 0 THEN
         INFILE.name = STRIP(INFILE.name)
      END
   ELSE
      PARSE UPPER PULL INFILE.name rest

   IF LENGTH(INFILE.name) = 0 THEN
      EXIT
   END

/* validate output filename */
IF LENGTH(OUTFILE.name) = 0 THEN
   DO
   SAY 'Please enter output file:'

   IF operatingSystem = 'CMS' THEN
      DO
      PARSE UPPER PULL OUTFILE.name
      IF LENGTH(OUTFILE.name) > 0 THEN
         OUTFILE.name = STRIP(OUTFILE.name)
      END
   ELSE
      PARSE UPPER PULL OUTFILE.name rest

   IF LENGTH(OUTFILE.name) = 0 THEN
      EXIT
   END

/* validate other parameters */
IF POS('CRLF',options) = 0 THEN
   crlf = 0
ELSE
   crlf = 1

IF POS('COMMENTS',options) = 0 THEN
   comments = 0
ELSE
   comments = 1

IF POS('REMOVELEADINGSPACES',options) = 0 & POS('RLS',options) = 0 THEN
   removeLeadingSpaces = 0
ELSE
   removeLeadingSpaces = 1

IF POS('REMOVETRAILINGSPACES',options) = 0 & POS('RTS',options) = 0 THEN
   removeTrailingSpaces = 0
ELSE
   removeTrailingSpaces = 1

IF POS('REMOVESEQUENCENUMS',options) = 0 & POS('RSN',options) = 0 THEN
   removeSequenceNums = 0
ELSE
   removeSequenceNums = 1

/* setup the assembler CSECT name */
IF LENGTH(csectName) > 8 THEN
   csectName = SUBSTR(csectName,1,8)

/* write out the assembler header */
OUTFILE.RECORDS.0 = 0
CALL writeLine("*" COPIES("-",65))
CALL writeLine("* CICS SupportPac CA8I..: Version 1.1")
CALL writeLine("* Generated on..........:" DATE() "-" TIME())
IF INFILE.name ^= 'INFILE' THEN
  DO
  CALL writeLine("* Input file............:" INFILE.name)
  CALL writeLine("* Output file...........:" OUTFILE.name)
  END
IF removeLeadingSpaces = 1 THEN
  CALL writeLine("* Remove leading spaces.: YES")
ELSE
  CALL writeLine("* Remove leading spaces.: NO")
IF removeTrailingSpaces = 1 THEN
  CALL writeLine("* Remove trailing spaces: YES")
ELSE
  CALL writeLine("* Remove trailing spaces: NO")
IF removeSequenceNums = 1 THEN
  CALL writeLine("* Remove sequence nums..: YES")
ELSE
  CALL writeLine("* Remove sequence nums..: NO")
IF crlf = 1 THEN
  CALL writeLine("* Add CR+LF to each line: YES")
ELSE
  CALL writeLine("* Add CR+LF to each line: NO")
IF comments = 1 THEN
  CALL writeLine("* Comments..............:",
                "YES - non-printable chars converted to '.'")
ELSE
  CALL writeLine("* Comments..............: NO")
CALL writeLine("* # bytes in input file.:")
totalBytesReadIndex = RESULT
CALL writeLine("* # bytes in prolog.....:")
totalBytesPrologIndex = RESULT
CALL writeLine("* # bytes in TEMPLATE...:")
totalBytesWrittenIndex = RESULT
CALL writeLine("*" COPIES("=",65))
CALL writeLine("         PUNCH ' ENTRY TEMPLATE'")
CALL writeLine(csectName 'CSECT')
CALL writeLine(csectName 'RMODE ANY')
CALL writeLine(csectName 'AMODE 31')
CALL writeLine("         DC CL9'"||csectName||"'")
CALL writeLine("         DC CL9'&SYSDATE'")
CALL writeLine("         DC CL5'&SYSTIME'")
CALL writeLine("* padding to align the end of TEMPLATE",
              "to an 8-byte boundary")
CALL writeLine("*")
boundaryPadding=RESULT
CALL writeLine("TEMPLATE EQU *")
CALL writeLine("         ENTRY TEMPLATE")
CALL writeLine("*" COPIES("-",65))

/* open the input file and read in all lines to a stem */
SAY 'DOCTMLT - opening input file:' INFILE.name
IF operatingSystem = 'TSO' & INFILE.name ^= 'INFILE' THEN
  DO
  "ALLOC DA("INFILE.name") F(INFILE) SHR REUSE"
  IF RC ^= 0 THEN
    EXIT 8
  INFILE.name = "INFILE"
  END
"EXECIO * DISKR" INFILE.name "(FINIS STEM INFILE.RECORDS.)"
IF INFILE.RECORDS.0 = 0 THEN
  DO
  SAY 'DOCTMLT - empty input file'
  EXIT 8
  END

/* create assembler table from the input file */
hex = ""
hexLength = 0
hexByteCount = 0
tableBoundary = 2
linkageBoundary = 8
maxBytesPerTableLine = 60
inputRecordIndex = 1
byteCount = 0
translateInput = ".<(+|&!$*);^-/,%_>?`:#@'="||'"'||"abcdefghijklmn"||,
                 "opqr~stuvwxyz{ABCDEFGHI}JKLMNOPQR\STUVWXYZ0123456789"
translateInput = TRANSLATE(XRANGE('00'X,'FF'X) , ,translateInput)

/* loop through the all input records */
DO FOREVER

  /* read records until at we have at least maxBytesPerTableLine  */
  /* characters of input                                          */
  DO WHILE (inputRecordIndex <= INFILE.RECORDS.0) &,
            (hexLength < maxBytesPerTableLine)

      /* keep a count of total bytes read from input records */
      byteCount = byteCount + LENGTH(INFILE.RECORDS.inputRecordIndex)

      /* convert input record string to hex string - stripping   */
      /* leading spaces, trailing spaces and sequence numbers as */
      /* required                                                */
      temp = INFILE.RECORDS.inputRecordIndex
      IF removeSequenceNums = 1 THEN
         temp = SUBSTR(temp,1,72)
      IF removeLeadingSpaces = 1 THEN
         temp = STRIP(temp,'L')
      IF removeTrailingSpaces = 1 THEN
         temp = STRIP(temp,'T')
      hex = hex||C2X(temp)

      /* add CR+LF characters to the end of the line as required */
      IF crlf = 1 THEN
         hex = hex||'0D25'

      /* calculate length of hex string */
      hexLength = LENGTH(hex)

      /* Align the CSECT to be on a 2 byte (half-word boundary) */
      temp = (((hexByteCount + hexLength)//(tableBoundary * 2)) /2)
      IF inputRecordIndex = INFILE.RECORDS.0 & temp > 0 THEN
        DO
        hex = hex||COPIES(C2X(' '),tableBoundary - temp)
        hexLength = LENGTH(hex)
        END

      /* step to next input record */
      inputRecordIndex = inputRecordIndex + 1
  END

  /* calculate the number of characters to write for this line */
  L = min(maxBytesPerTableLine, hexLength)

  /* write comments string as required */
  IF comments = 1 THEN
     CALL writeLine("*     "||,
       TRANSLATE(X2C(LEFT(hex,L)), ,translateInput,'.'))

  /* write hex string */
  CALL writeLine(" DC X'"||LEFT(hex,L)||"'")

  /* calculate remaining hex string */
  hexLength = hexLength - L
  hexByteCount = hexByteCount + L
  hex = right(hex,hexLength)

  /* leave loop if input file completely processed */
  IF (inputRecordIndex > INFILE.RECORDS.0) & hexLength = 0 THEN
     LEAVE
END

CALL writeLine("*" COPIES("-",65))
CALL writeLine("         END TEMPLATE")

/* The MVS linkage editor forces load modules to be multiples of 8    */
/* bytes (double word boundaries) and will append x'00' characters as */
/* required to the program object which will be included in the       */
/* template by CICS. It is therefore preferable to append spaces      */
/* (x'40') now to correctly allign to this boundary.                  */
prologSize = 9 + 9 + 5
temp = linkageBoundary - ((prologSize +,
                           (hexByteCount / 2))//linkageBoundary)
IF temp > 0 THEN
  DO
  OUTFILE.RECORDS.boundaryPadding="         DC X'"||,
    COPIES(C2X(' '),temp)||"'"
  prologSize = prologSize + temp
  END

/* fill in the output assembler total fields */
OUTFILE.RECORDS.totalBytesPrologIndex =,
                   OUTFILE.RECORDS.totalBytesPrologIndex prologSize
OUTFILE.RECORDS.totalBytesReadIndex =,
                   OUTFILE.RECORDS.totalBytesReadIndex byteCount
OUTFILE.RECORDS.totalBytesWrittenIndex =,
                   OUTFILE.RECORDS.totalBytesWrittenIndex hexByteCount/2

/* open the output file and write the output stem */
SAY 'DOCTMLT - writing file:' OUTFILE.name
IF operatingSystem = 'TSO' & OUTFILE.name ^= 'OUTFILE' THEN
   DO
   "ALLOC DA("OUTFILE.name") F(OUTFILE) OLD"
   IF RC ^= 0 THEN
     EXIT 8
   OUTFILE.name = 'OUTFILE'
   END
"EXECIO * DISKW" OUTFILE.name " (FINIS STEM OUTFILE.RECORDS.)"

/* exit the program */
EXIT


/* subroutine to add a line of text to the output stem */
writeLine:
PARSE ARG s
r = OUTFILE.RECORDS.0 + 1
OUTFILE.RECORDS.r = s
OUTFILE.RECORDS.0 = r
RETURN r


HelpDocumentation:
SAY 'SYNTAX: DOCTMLT <INFILE> <OUTFILE> <CSECT> ( <options>           '
SAY '                                                                 '
SAY 'where;                                                           '
SAY '                                                                 '
SAY '   INFILE   = input HTML template file                           '
SAY '   OUTFILE  = output S390 Assembler file - if this file already  '
SAY '              exists it will be overwritten without warning      '
SAY '   CSECT    = the name of the assembler csect name               '
SAY '                                                                 '
SAY 'and the following are parameters are optional;                   '
SAY '                                                                 '
SAY '   CRLF     = append CR+LR characters to each input line         '
SAY '   COMMENTS = add comment lines                                  '
SAY '   REMOVELEADINGSPACES  = remove leading spaces from input lines '
SAY '   REMOVETRAILINGSPACES = remove trailing spaces from input lines'
SAY '   REMOVESEQUENCENUMS   = remove sequnce nums in cols 73-80      '
SAY '                                                                 '
SAY 'The DOCTMLT EXEC will read the specified INFILE and generate the '
SAY 'OUTFILE which will be an MVS assembler data only program with DC '
SAY 'statements containing the hex byte equivalent of the input file. '
EXIT
